#ifndef _DISTANCETRANSFORM_H_
#define _DISTANCETRANSFORM_H_

#include "Image.hpp"

#include <cassert>
#include <limits>
#include <cmath>

/**
 * @brief Compute distance transform
 * @note  Assumes that imOutput is of the proper size
 */
template <class T>
void cppComputeDistanceTransformImage(const Image<T>& imInput, Image<T>& imOutput)
{
  static const float thresholdVal = 1.0f;

  assert( imInput.getTotalSize() == imOutput.getTotalSize() );

  const size_t dx = imInput.getDx();
  const size_t dy = imInput.getDy();
  
  const float* dataPtrInput = imInput.getDataPtr();
  float* dataPtrOutput = imOutput.getDataPtr();

  //Outer loop (for every posistion in output image)
  for(size_t x=0; x < dx; ++x)
  {
    for(size_t y=0; y < dy; ++y)
    {
      //Inner loop (... get the closet position above threshold in input image)
      T minDist = std::numeric_limits<T>::max();
      for(size_t ox=0; ox < dx; ++ox)
      {
        for(size_t oy=0; oy < dy; ++oy)
        {
          if( dataPtrInput[dy*oy + ox] >= thresholdVal )
          {
            float dist = std::sqrt( 
              static_cast<float>((ox-x)*(ox-x)) + 
              static_cast<float>((oy-y)*(oy-y)) );
            if( dist < minDist )
            {
              minDist = dist;
            }
          }
        }
      }
      dataPtrOutput[dy*y + x] = minDist;
    }
  }  

}

#endif _DISTANCETRANSFORM_H_
