#ifndef _Image_H_
#define _Image_H_

#include <vector>
#include <exception>
#include <string>
#include <sstream>
#include <cassert>

template <class T>
class Image
{
public:
  Image(size_t dx, size_t dy, const T& initVal)
  {
    m_dx = dx;
    m_dy = dy;
    
    m_ptrData = new T[getTotalSize()];
    fillDataWith(initVal);
  }

  ///@note Be careful, data will NOT be initialized with this construct
  Image(size_t dx, size_t dy)
  {
    m_dx = dx;
    m_dy = dy;

    m_ptrData = new T[getTotalSize()];
  }

  ~Image()
  {
    delete[] m_ptrData;
  }
  
  size_t getDx() const
  {
    return m_dx;
  }

  size_t getDy() const
  {
    return m_dy;
  }

  size_t getTotalSize() const
  {
    return m_dx*m_dy;
  }

  T* getDataPtr()
  {
    return m_ptrData;
  }

  T* getDataPtr() const
  {
    return m_ptrData;
  }

  void safeSet(size_t x, size_t y, const T& val)
  {
    size_t pos = m_dy*y + x;
    if(pos >= getTotalSize())
    {
      throw std::exception("safeSet() out of bounds");
    }
    m_ptrData[pos] = val;
  }

  T safeGet(size_t x, size_t y)
  {
    size_t pos = m_dy*y + x;
    if(pos >= getTotalSize())
    {
      throw std::exception("safeSet() out of bounds");
    }
    return m_ptrData[pos];
  }

  static bool isEqual(const Image<T>& lhs, const Image<T>& rhs)
  {
    assert(lhs.getTotalSize() == rhs.getTotalSize());
    
    bool isEqual = true;
    for(size_t i = 0; i < lhs.getTotalSize(); ++i)
    {
      //@tag Mention report *almost* --> SOME difference
      if(!floatIsAlmostEqual(lhs.getDataPtr()[i], rhs.getDataPtr()[i]))
      {
        float f_lhs = lhs.getDataPtr()[i];
        float f_rhs = rhs.getDataPtr()[i];
        isEqual = false;
        break;
      }
    }

    return isEqual;
  }

private:
  void fillDataWith(T val)
  {
    size_t totalSize = getTotalSize();
    for(size_t i=0; i < getTotalSize(); ++i)
    {
      m_ptrData[i] = val;
    }
  }

  T* m_ptrData;

  size_t m_dx;
  size_t m_dy;
};

template<class T>
std::string ImageToStr(const Image<T>& volInput)
{
  std::stringstream ss;

  size_t dx = volInput.getDx();
  size_t dy = volInput.getDy();
  float* dataPtr = volInput.getDataPtr();

  for(size_t y=0; y < dy; ++y)
  {
    for(size_t x=0; x < dx; ++x)
    {
      float res_ptr = dataPtr[dy*y + x];
      ss<<res_ptr<<" ";
    }
    ss<<std::endl;
  }

  return ss.str();
}


#endif